@@ -112,6 +112,7 @@ void tvterm_init(TVterm* p, TTerm* pt, u_int hw, u_int hh, TCaps *caps, const ch
 	p->otherCS = NULL;
 #endif
 	p->caps = caps;
+	p->altCs = FALSE;
 	tvterm_set_default_encoding(p, en);
 }
 
@@ -432,6 +433,7 @@ void tvterm_set_default_invoke_and_designate(TVterm* p)
 	p->utf8remain = 0;
 	p->ucs2ch = 0;
 #endif
+	p->altCs = FALSE;
 }
 
 void tvterm_start(TVterm* p)
@@ -449,6 +451,7 @@ void tvterm_start(TVterm* p)
 	p->ins = FALSE;
 	p->wrap = FALSE;
 	p->active = TRUE;
+	p->altCs = FALSE;
 
 	/* ISO-2022 */
 	p->escSignature = 0;
@@ -772,7 +775,12 @@ tvterm_put_utf8_char(TVterm *p, u_char ch)
 	} else {
 		/* XXX: support UCS2 only */
 	}
-	rev = tvterm_put_uchar(p, p->ucs2ch);
+	if (p->altCs 
+	    && (p->ucs2ch < 127)
+	    && (IS_GL_AREA(p, (u_char)(p->ucs2ch & 0x0ff))))
+		rev = tvterm_put_normal_char(p, (p->ucs2ch & 0x0ff));
+	else
+		rev = tvterm_put_uchar(p, p->ucs2ch);
 	if (rev < 0) {
 		p->ucs2ch >>= 6;
 		if (p->ucs2ch)
@@ -1066,6 +1074,7 @@ void tvterm_esc_set_attr(TVterm* p, int col)
 		tvterm_re_invoke_gx(p, &(p->gl));
 		tvterm_re_invoke_gx(p, &(p->gr));
 		p->tgl = p->gl; p->tgr = p->gr;
+		p->altCs = FALSE;
 		break;
 	case 11:	/* smacs, smpch */
 		if (acsIdx == 0) {
@@ -1076,6 +1085,7 @@ void tvterm_esc_set_attr(TVterm* p, int col)
 			tvterm_re_invoke_gx(p, &(p->gl));
 			tvterm_re_invoke_gx(p, &(p->gr));
 			p->tgl = p->gl; p->tgr = p->gr;
+			p->altCs = TRUE;
 		}
 		break;
 	default: tpen_set_color(&(p->pen), col);
