@@ -1,5 +1,6 @@
 /*
  * JFBTERM -
+ * Copyright (c) 2003 Fumitoshi UKAI <ukai@debian.or.jp>
  * Copyright (C) 1999 Noritoshi MASUICHI (nmasu@ma3.justnet.ne.jp)
  *
  * KON2 - Kanji ON Console -
@@ -29,6 +30,7 @@
  * 
  */
 
+#define __USE_STRING_INLINES
 #include	<stdio.h>
 #include	<stdlib.h>
 #include	<string.h>
@@ -88,185 +90,34 @@ static inline void	KanjiAdjust(TVterm* p, u_int *x, u_int *y)
 }
 
 /*---------------------------------------------------------------------------*/
-static inline void bmove(void *dst, void *src, int n)
-{
-#ifdef __i386__
-	int dummy1;
-	int dummy2;
-	int dummy3;
-	__asm__ __volatile__ (
-		"cld\n\t"
-		"rep\n\t"
-		"movsb\n\t"
-		:"=c" (dummy1), "=D" (dummy2), "=S" (dummy3)
-		:"0" (n), "1" ((long)dst), "2" ((long)src)
-	);
-#else
-    memmove(dst,src,n);
-#endif
-}
-
 static inline void brmove(void *dst, void *src, int n)
 {
-#ifdef __i386__
-	int dummy1;
-	int dummy2;
-	int dummy3;
-	__asm__ __volatile__ (
-		"std\n\t"
-		"rep\n\t"
-		"movsb\n\t"
-		:"=c" (dummy1), "=D" (dummy2), "=S" (dummy3)
-		:"0" (n), "1" ((long)dst), "2" ((long)src)
-	);
-#else
 	memmove((void*)((char*)dst-n),(void*)((char*)src-n),n);
-#endif
 }
 
-static inline void bzero2(void *head, int n)
-{
-#ifdef __i386__
-	int dummy1;
-	int dummy2;
-	__asm__ __volatile__ (
-		"cld\n\t"
-		"rep\n\t"
-		"stosb"
-		:"=c" (dummy1), "=D" (dummy2)
-		:"a" (0), "0" (n), "1" ((long)head)
-	);
-#else
-    memset(head,0,n);
-#endif
-}
-
-static inline void memset2(void *head, u_char ch, int n)
-{
-#ifdef __i386__
-	int dummy1;
-	int dummy2;
-	__asm__ __volatile__ (
-		"cld\n\t"
-		"rep\n\t"
-		"stosb"
-		:"=c" (dummy1), "=D" (dummy2)
-		:"a" (ch), "0" (n), "1" ((long)head)
-	);
-#else
-    memset(head,ch,n);
-#endif
-}
-
-static inline void lmove(void *dst, void *src, int n)
-{
-#ifdef __i386__
-	int dummy1;
-	int dummy2;
-	int dummy3;
-	__asm__ __volatile__ (
-		"cld\n\t"
-		 "rep\n\t"
-		 "movsl\n\t"
-		:"=c" (dummy1), "=D" (dummy2), "=S" (dummy3)
-		:"0" (n>>2), "1" ((long)dst), "2" ((long)src)
-	);
-#else
-    memmove(dst,src,n);
-#endif
-}
-
-static inline void lzero(void *head, int n)
-{
-#ifdef __i386__
-	int dummy1;
-	int dummy2;
-	__asm__ __volatile__ (
-		"cld\n\t"
-		"rep\n\t"
-		"stosl"
-		:"=c" (dummy1), "=D" (dummy2)
-		:"a" (0), "0" (n>>2), "1" ((long)head)
-	);
-#else
-    memset(head,0,n);
-#endif
-}
-
-static inline void lset(void *head, u_char ch, int n)
-{
-#ifdef __i386__
-	int dummy1;
-	int dummy2;
-	__asm__ __volatile__ (
-		"cld\n\t"
-		"rep\n\t"
-		"stosl"
-		:"=c" (dummy1), "=D" (dummy2)
-		:"a" (ch<<24|ch<<16|ch<<8|ch), "0" (n>>2), "1" ((long)head)
-	);
-#else
-    memset(head,ch,n);
-#endif
-}
-
-
 static inline void blatch(void *head, int n)
 {
-#ifdef __i386__
-	int dummy1;
-	int dummy2;
-	__asm__ __volatile__ (
-		"\t clc\n"
- 		"1:\n"
- 		"\t andb %%bl, (%%eax)\n"
- 		"\t incl %%eax\n"
- 		"\t loop 1b\n"
-		: "=b" (dummy1), "=c" (dummy2)
-		: "eax" ((long)head), "0" (0x7F), "1" (n)
-	);
-#else
  	unsigned char *c = (unsigned char*)head;
-	for(; c < (unsigned char*)head+n ; c++) {
+	unsigned char *e = (unsigned char*)head + n;
+	for(; c < e ; c++) {
 		*c &= 0x7f;
 	}
-#endif
 }
 
 static inline void llatch(void *head, int n)
 {
-#ifdef __i386__
-	int dummy1;
-	int dummy2;
-	__asm__ __volatile__ (
-		"\t clc\n"
- 		"1:\n"
- 		"\t andl %%ebx, (%%eax)\n"
- 		"\t addl $4, %%eax\n"
- 		"\t loop 1b\n"
-		: "=b" (dummy1), "=c" (dummy2)
-		: "eax" ((long)head), "0" (0x7F7F7F7F), "1" (n>>2)
-	);
-#else
 	unsigned int *l = (unsigned int*)head;
-	for(; l < (unsigned int*) head+(n>>2) ; l++) {
+	unsigned int *e = (unsigned int*)head + (n>>2);
+	for(; l < e ; l++) {
 		*l &= 0x7f7f7f7f;
 	}
-#endif
 }
 
-static inline void tvterm_bmove(TVterm* p, int dst, int src, int n)
+static inline void tvterm_move(TVterm* p, int dst, int src, int n)
 {
-	bmove(p->text+dst, p->text+src, n*sizeof(u_int));
-	bmove(p->attr+dst, p->attr+src, n);
-	bmove(p->flag+dst, p->flag+src, n);
-}
-
-static inline void tvterm_lmove(TVterm* p, int dst, int src, int n)
-{
-	lmove(p->text+dst, p->text+src, n*sizeof(u_int));
-	lmove(p->attr+dst, p->attr+src, n);
-	lmove(p->flag+dst, p->flag+src, n);
+	memmove(p->text+dst, p->text+src, n*sizeof(u_int));
+	memmove(p->attr+dst, p->attr+src, n);
+	memmove(p->flag+dst, p->flag+src, n);
 }
 
 static inline void tvterm_brmove(TVterm* p, int dst, int src, int n)
@@ -278,17 +129,11 @@ static inline void tvterm_brmove(TVterm* p, int dst, int src, int n)
 
 static inline void tvterm_clear(TVterm* p, int top, int n)
 {
-	bzero2(p->text+top, n*sizeof(u_int));
-	bzero2(p->flag+top, n);
-	memset2(p->attr+top, p->pen.bcol, n);
+	bzero(p->text+top, n*sizeof(u_int));
+	bzero(p->flag+top, n);
+	memset(p->attr+top, (p->pen.bcol<<4), n);
 }
 
-static inline void tvterm_lclear(TVterm* p, int top, int n)
-{
-	lzero(p->text+top, n*sizeof(u_int));
-	lzero(p->flag+top, n);
-	lset(p->attr+top, p->pen.bcol, n);
-}
 /*---------------------------------------------------------------------------*/
 void tvterm_delete_n_chars(TVterm* p, int n)
 {
@@ -298,7 +143,7 @@ void tvterm_delete_n_chars(TVterm* p, int n)
 	addr = tvterm_coord_to_index(p, p->pen.x, p->pen.y);
 	dx = p->xcap - p->pen.x - n;
 	
-	tvterm_bmove(p, addr, addr+n, dx);
+	tvterm_move(p, addr, addr+n, dx); /* bmove */
 	blatch(p->flag+addr, dx);
 	
 	addr = tvterm_coord_to_index(p, p->xcap-n, p->pen.y);
@@ -331,12 +176,12 @@ static void tvterm_scroll_up_n_lines(TVterm* p, int n)
 		p->textHead -= p->tsize;
 		n = p->tsize - h;
 		if (p->textHead) {
-			tvterm_lclear(p, 0, p->textHead);
+			tvterm_clear(p, 0, p->textHead); /* lclear */
 		}
 	} else  {
 		n = p->textHead - h;
 	}
-	tvterm_lclear(p, h, n);
+	tvterm_clear(p, h, n); /* lclear */
 }
 
 static void tvterm_scroll_down_n_lines(TVterm* p, int n)
@@ -349,13 +194,13 @@ static void tvterm_scroll_down_n_lines(TVterm* p, int n)
 	if (p->textHead < 0) {
 		p->textHead += p->tsize;
 		if (h) {
-			tvterm_lclear(p, 0, h);
+			tvterm_clear(p, 0, h); /* lclear */
 		}
 		n = p->tsize - p->textHead;
 	} else {
 		n = h - p->textHead;
 	}
-	tvterm_lclear(p, h, n);
+	tvterm_clear(p, h, n); /* lclear */
 }
 #endif
 
@@ -470,6 +315,8 @@ void tvterm_refresh(TVterm* p)
 				gFontsWidth * x, gFontsHeight * y,
 				gFontsWidth * w,
 				gFontsHeight, bc);
+			if (chlw == 0)
+				continue;
 			gFramebuffer.cap.overlay(&gFramebuffer,
 				gFontsWidth * x, gFontsHeight * y,
 				glyph, gw, pf->height, pf->bytew, fc);
@@ -608,7 +455,7 @@ void tvterm_text_clear_all(TVterm* p)
 
 	for (y = 0; y < p->ymax; y++) {
 		a = tvterm_coord_to_index(p, 0, y);
-		tvterm_lclear(p, a, p->xcap4);
+		tvterm_clear(p, a, p->xcap4); /* lclear */
 	}
 	p->textClear = TRUE;
 }
@@ -684,7 +531,7 @@ static void tvterm_text_clean_band(TVterm* p, u_int top, u_int btm)
 	
 	for (y = top; y < btm; y ++) {
 		a = tvterm_coord_to_index(p, 0, y);
-		tvterm_lclear(p, a, p->xcap4);
+		tvterm_clear(p, a, p->xcap4); /* lclear */
 		/* needless to latch */
 	}
 }
@@ -708,7 +555,7 @@ void tvterm_text_move_down(TVterm* p, int top, int btm, int line)
 		for (n = btm-1; n >= top + line; n --) {
 			dst = tvterm_coord_to_index(p, 0, n);
 			src = tvterm_coord_to_index(p, 0, n-line);
-			tvterm_lmove(p, dst, src, p->xcap4);
+			tvterm_move(p, dst, src, p->xcap4); /* lmove */
 			llatch(p->flag + dst, p->xcap4);
 		}
 		tvterm_text_clean_band(p, top, top + line);
@@ -734,7 +581,7 @@ void tvterm_text_move_up(TVterm* p, int top, int btm, int line)
 		for (n = top; n < btm - line; n ++) {
 			dst = tvterm_coord_to_index(p, 0, n);
 			src = tvterm_coord_to_index(p, 0, n+line);
-			tvterm_lmove(p, dst, src, p->xcap4);
+			tvterm_move(p, dst, src, p->xcap4); /* lmove */
 			llatch(p->flag + dst, p->xcap4);
 		}
 		tvterm_text_clean_band(p, btm - line, btm);
